(*
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License.        *
 *                                                                         *
 ***************************************************************************
*)


unit fWAZITUStat;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, inifiles,
  ExtCtrls, Grids, Buttons, StdCtrls;

type
  TStatType = (tsWAZ,tsITU,tsWAC,tsWAS);
  TCfmType  = (tcQSL,tcQSLLoTW,tcLoTW);

type

  { TfrmWAZITUStat }

  TfrmWAZITUStat = class(TForm)
    btnClose: TButton;
    btnHTMLExport: TButton;
    btnRefresh: TButton;
    btnSelectProfile: TButton;
    btnShowSationList: TButton;
    cmbMode: TComboBox;
    dlgSave: TSaveDialog;
    edtProfiles: TEdit;
    grdStat: TStringGrid;
    grdSumStat: TStringGrid;
    Label1: TLabel;
    Label2: TLabel;
    Panel1: TPanel;
    procedure btnSelectProfileClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnHTMLExportClick(Sender: TObject);
    procedure btnRefreshClick(Sender: TObject);
    procedure btnShowSationListClick(Sender: TObject);
  private
    gmode : String;
    aStates : Array [1..50] of String;
    procedure CreateWAZStat;
    procedure CreateWACStat;
    procedure CreateWASStat;
    procedure LoadBandsSettings;
    procedure CreateSummary;
    procedure ShowBandChar(band : String;y : Integer;bchar : Char);
  public
    StatType : TStatType;
    CfmType  : TCfmType;
    procedure ExportToHTML(htmlfile : String);
  end; 

var
  frmWAZITUStat: TfrmWAZITUStat;

implementation

{ TfrmWAZITUStat }
uses dUtils,dData, fQTHProfiles, fShowStations, uMyIni;

procedure TfrmWAZITUStat.ShowBandChar(band : String;y : Integer;bchar : Char);
var
  p : Integer;
begin
  if (y > grdStat.RowCount) or (y < 1) then
  begin
    Writeln('bChar: ',bchar);
    Writeln('band:  ',band);
    Writeln('y:     ',y);
    exit;
  end;
  p := dmUtils.GetBandPos(band);
  if p > -1 then
    grdStat.Cells[p+1,y] := bchar
end;

procedure TfrmWAZITUStat.CreateWAZStat;
var
  zone : Integer;
  i    : Integer = 1;
  y    : Integer;
begin

  for i:=0 to grdStat.RowCount-1 do
    for y:=0 to grdStat.ColCount-1 do
      grdStat.Cells[y,i] := '';

  LoadBandsSettings;

  if StatType = tsWAZ then
    Caption := 'WAZ statistic'
  else
    Caption := 'ITU statistic';

  if StatType = tsWAZ then
  begin
    Caption := 'WAZ statistic';
    grdStat.Cells[0,0] := 'WAZ'
   end
  else begin
    Caption := 'ITU statistic';
    grdStat.Cells[0,0] := 'ITU'
  end;

  dmData.Q.Close();
  if StatType = tsWAZ then
  begin
    if gmode <> '' then
      dmData.Q.SQL.Text := 'select waz,band from cqrlog_main where '+gmode+' group by waz,band order by waz'
    else
      dmData.Q.SQL.Text := 'select waz,band from cqrlog_main group by waz,band order by waz'
  end
  else begin
    if gmode <> '' then
      dmData.Q.SQL.Text := 'select itu,band from cqrlog_main where '+gmode+' group by itu,band order by itu'
    else
      dmData.Q.SQL.Text := 'select itu,band from cqrlog_main group by itu,band order by itu'
  end;

  if dmData.DebugLevel >=1 then
    Writeln(dmData.Q.SQL.Text);
  if dmData.trQ.Active then
    dmData.trQ.Rollback;
  dmData.trQ.StartTransaction;
  dmData.Q.Open();
  dmData.Q.First;

  if StatType = tsWAZ then
  begin
    grdStat.RowCount := 41;
    for i:=1 to 40 do
      grdStat.Cells[0,i] := IntToStr(i);
  end
  else begin
    grdStat.RowCount := 76;
    for i:=1 to 75 do
      grdStat.Cells[0,i] := IntToStr(i);
  end;

  while not dmData.Q.Eof do
  begin
    zone := dmData.Q.Fields[0].AsInteger;
    while not (zone <> dmData.Q.Fields[0].AsInteger) and (not dmData.Q.Eof) do
    begin
      if StatType = tsWAZ then
      begin
        if zone > 40 then
        begin
          dmData.Q.Next;
          Continue
        end;
      end
      else begin
        if zone > 75 then
        begin
          dmData.Q.Next;
          Continue
        end;
      end;
      if (zone <> 0) then
        ShowBandChar(dmData.Q.Fields[1].AsString,zone,'X');
      dmData.Q.Next
    end
  end;

  if (CfmType = tcQSLLoTW) or (CfmType = tcLoTW) then
  begin
    dmData.Q.Close();
    if StatType = tsWAZ then
    begin
      if gmode <> '' then
          dmData.Q.SQL.Text := 'select waz,band from cqrlog_main WHERE lotw_qslr=' +
                             QuotedStr('L') + ' AND ' + gmode +' group by waz,band order by waz'
      else
        dmData.Q.SQL.Text := 'select waz,band from cqrlog_main WHERE lotw_qslr=' +
                             QuotedStr('L') +' group by waz,band order by waz'
    end
    else begin
      if gmode <> '' then
        dmData.Q.SQL.Text := 'select itu,band from cqrlog_main WHERE lotw_qslr=' +
                             QuotedStr('L') + ' AND ' + gmode +' group by itu,band order by itu'
      else
        dmData.Q.SQL.Text := 'select itu,band from cqrlog_main WHERE lotw_qslr=' +
                             QuotedStr('L') +' group by itu,band order by itu'
    end;
    if dmData.DebugLevel >=1 then
      Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      zone := dmData.Q.Fields[0].AsInteger;
      while not (zone <> dmData.Q.Fields[0].AsInteger) and (not dmData.Q.Eof) do
      begin
        if StatType = tsWAZ then
        begin
          if zone > 40 then
          begin
            dmData.Q.Next;
            Continue
          end;
        end
        else begin
          if zone > 75 then
          begin
            dmData.Q.Next;
            Continue
          end;
        end;
        if (zone <> 0) then
          ShowBandChar(dmData.Q.Fields[1].AsString,zone,'L');
        dmData.Q.Next
      end
    end;
    dmData.Q.Close()
  end;
  if (CfmType <> tcLoTW) then
  begin
    dmData.Q.Close();
    if StatType = tsWAZ then
    begin
      if gmode <> '' then
        dmData.Q.SQL.Text := 'select waz,band from cqrlog_main WHERE qsl_r=' +
                             QuotedStr('Q') + ' AND '+ gmode +' group by waz,band order by waz'
      else
        dmData.Q.SQL.Text := 'select waz,band from cqrlog_main WHERE qsl_r=' +
                             QuotedStr('Q') +' group by waz,band order by waz'
    end
    else begin
      if gmode <> '' then
        dmData.Q.SQL.Text := 'select itu,band from cqrlog_main WHERE qsl_r=' +
                             QuotedStr('Q') + 'AND' + gmode +' group by itu,band order by itu'
      else
        dmData.Q.SQL.Text := 'select itu,band from cqrlog_main WHERE qsl_r=' +
                             QuotedStr('Q') +' group by itu,band order by itu'
    end;
    if dmData.DebugLevel >=1 then
      Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      zone := dmData.Q.Fields[0].AsInteger;
      while not (zone <> dmData.Q.Fields[0].AsInteger) and (not dmData.Q.Eof) do
      begin
        if StatType = tsWAZ then
        begin
          if zone > 40 then
          begin
            dmData.Q.Next;
            Continue
          end;
        end
        else begin
          if zone > 75 then
          begin
            dmData.Q.Next;
            Continue
          end;
        end;
        if (zone <> 0) then
          ShowBandChar(dmData.Q.Fields[1].AsString,zone,'Q');
        dmData.Q.Next
      end
    end;
    dmData.Q.Close()
  end;
  dmData.trQ.RollBack;
  CreateSummary
end;

procedure TfrmWAZITUStat.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  dmUtils.SaveForm(self);
  cqrini.WriteString('WAZITUStat','profiles'+IntToStr(ord(StatType)),edtProfiles.Text);
  cqrini.WriteInteger('WAZITUStat','mode'+IntToStr(ord(StatType)),cmbMode.ItemIndex);
  cqrini.WriteInteger('WAZITUStat','width'+IntToStr(ord(StatType)),grdStat.ColWidths[0])
end;

procedure TfrmWAZITUStat.btnSelectProfileClick(Sender: TObject);
begin
  with TfrmQTHProfiles.Create(self) do
  try
    SelectingProfiles;
    ShowModal;
    if ModalResult = mrOK then
    begin
      if edtProfiles.Text = '' then
        edtProfiles.Text := dmData.qProfiles.Fields[1].AsString
      else begin
        if edtProfiles.Text[Length(edtProfiles.Text)] <> ';' then
          edtProfiles.Text := edtProfiles.Text + ';'+dmData.qProfiles.Fields[1].AsString
        else
          edtProfiles.Text := edtProfiles.Text + dmData.qProfiles.Fields[1].AsString
      end
    end
  finally
    Free
  end
end;

procedure TfrmWAZITUStat.FormCreate(Sender: TObject);
begin
  dmUtils.LoadWindowPos(self);
  dmUtils.LoadFontSettings(self)
end;

procedure TfrmWAZITUStat.FormShow(Sender: TObject);
var
  w : Integer = 0;
begin
  gmode := '';
  dmUtils.InsertModes(cmbMode);
  cmbMode.Items.Insert(0,'ALL');
  cmbMode.Items.Insert(1,'SSB+CW');
  edtProfiles.Text  := cqrini.ReadString('WAZITUStat','profiles'+IntToStr(ord(StatType)),'');
  cmbMode.ItemIndex := cqrini.ReadInteger('WAZITUStat','mode'+IntToStr(ord(StatType)),0);
  w                 := cqrini.ReadInteger('WAZITUStat','width'+IntToStr(ord(StatType)),0);
  if w = 0 then
  begin
    if StatType = tsWAS then
      grdStat.ColWidths[0] := 140
    else
      grdStat.ColWidths[0] := 40
  end
  else
    grdStat.ColWidths[0] := w;
  if (edtProfiles.Text <> '') or (cmbMode.ItemIndex>0) then
  begin
    btnRefresh.Click;
    exit
  end;
  cmbMode.ItemIndex := 0;
  case StatType of
    tsWAZ,tsITU : CreateWAZStat;
    tsWAC   : CreateWACStat;
    tsWAS   : CreateWASStat
  end
end;

procedure TfrmWAZITUStat.btnHTMLExportClick(Sender: TObject);
begin
  dlgSave.DefaultExt := '.html';
  dlgSave.Filter := 'html|*.html;*.HTML';
  if dlgSave.Execute then
  begin
    ExportToHTML(dlgSave.FileName);
  end;
end;

procedure TfrmWAZITUStat.btnRefreshClick(Sender: TObject);
var
  tmp : String = '';
  a   : TExplodeArray;
  i   : Integer = 0;
begin
  case cmbMode.ItemIndex of
    0 : gmode := '';
    1 : gmode := ' ((mode = '+QuotedStr('CW')+') or (mode = '+QuotedStr('SSB')+')) '
   else
     gmode := ' (mode = '+QuotedStr(cmbMode.Text)+') '
  end;
  if edtProfiles.Text <> '' then
  begin
    tmp := '';
    SetLength(a,0);
    if Pos(';',edtProfiles.Text) > 0 then
    begin
      a := dmUtils.Explode(';',edtProfiles.Text);
      for i:=0 to Length(a)-1 do
        tmp := tmp + ' (profile = '+a[i]+') OR';
      tmp := '('+copy(tmp,1,Length(tmp)-2)+')'
    end
    else
      tmp := ' (profile = '+edtProfiles.Text+') ';
    if gmode = '' then
      gmode := tmp
    else
      gmode := gmode + ' AND ' + tmp
  end;
  case StatType of
    tsWAZ,tsITU : CreateWAZStat;
    tsWAC   : CreateWACStat;
    tsWAS   : CreateWASStat
  end
end;

procedure TfrmWAZITUStat.btnShowSationListClick(Sender: TObject);
var
  sql : String;
  l   : TStringList;
  b   : String = '';
  oldb : String = '';
  tmp  : String;
  qslr : String = '';
begin
  l := TStringList.Create;
  try
    case CfmType of
      tcQSL : qslr := '(qsl_r = ' + QuotedStr('Q')+')';
      tcQSLLoTW : qslr := '((qsl_r = ' + QuotedStr('Q')+') or (lotw_qslr='+QuotedStr('L')+'))';
      tcLoTW : qslr := '(lotw_qslr = '+QuotedStr('L')+')'
     end;

    case StatType of
      tsWAZ   : begin
              if gmode = '' then
                sql := 'select main.callsign, main.freq,main.mode,main.waz from ( '+
                       'select waz,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(waz <> 0) and '+ qslr +
                       'group by waz,band,qsl_r order by waz,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),waz'
              else
                sql := 'select main.callsign,main.freq,main.mode,main.waz from ( '+
                       'select waz,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(waz <> 0) and '+ qslr + ' and ' + gmode +' '+
                       'group by waz,band,qsl_r order by waz,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),waz';
              l.Add('WAZ')
            end;//waz
      tsITU   : begin
              if gmode = '' then
                sql := 'select main.callsign, main.freq,main.mode,main.itu from ( '+
                       'select itu,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(itu <> 0) and '+ qslr +
                       'group by itu,band,qsl_r order by itu,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),itu'
              else
                sql := 'select main.callsign,main.freq,main.mode,main.itu from ( '+
                       'select itu,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(itu <> 0) and '+ qslr + ' and ' + gmode +' '+
                       'group by itu,band,qsl_r order by itu,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),itu';
              l.Add('ITU')
            end;//itu
      tsWAC  : begin
              if gmode = '' then
                sql := 'select main.callsign, main.freq,main.mode,main.cont from ( '+
                       'select cont,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(cont <> '+QuotedStr('')+') and '+ qslr +
                       'group by cont,band,qsl_r order by cont,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),cont'
              else
                sql := 'select main.callsign, main.freq,main.mode,main.cont from ( '+
                       'select cont,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(cont <> '+QuotedStr('')+') and '+ qslr + ' and ' + gmode +' '+
                       'group by cont,band,qsl_r order by cont,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by convert(freq,signed),cont';
              l.Add('WAC')
            end;//wac
      tsWAS   : begin
              if gmode = '' then
                sql := 'select callsign,freq,mode,state from cqrlog_main '+
                        ' where (state <> '+QuotedStr('')+') and ((dxcc_ref = '+QuotedStr('W(USA)')+') or (dxcc_ref='+
                        QuotedStr('KL')+') or (dxcc_ref='+QuotedStr('KH6')+')) and '+ qslr +' order by convert(freq,signed),state'
              else
                sql := 'select callsign,freq,mode,state from grb_cqrlog_main'+
                        ' where (state <> '+QuotedStr('')+') and ((dxcc_ref = '+QuotedStr('W(USA)')+') or (dxcc_ref='+
                        QuotedStr('KL')+') or (dxcc_ref='+QuotedStr('KH6')+')) and '+ qslr +' AND '+gmode+
                        'order by convert(freq,signed),state';
              {if gmode = '' then
                sql := 'select subsel.id_cqrlog_main, main.callsign, main.freq,main.mode,main.state from ( '+
                       'select state,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(state <> '+QuotedStr('')+') and ((dxcc_ref = '+QuotedStr('W(USA)')+') or (dxcc_ref='+
                       QuotedStr('KL')+') or (dxcc_ref='+QuotedStr('KH6')+')) and '+ qslr +
                       'group by state,band,qsl_r order by state,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by freq,state'
              else
                sql := 'select subsel.id_cqrlog_main, main.callsign, main.freq,main.mode,main.state from ( '+
                       'select state,band,qsl_r,max(a.id_cqrlog_main) as id_cqrlog_main from cqrlog_main a where '+
                       '(state <> '+QuotedStr('')+') and ((dxcc_ref = '+QuotedStr('W(USA)')+') or (dxcc_ref='+
                       QuotedStr('KL')+') or (dxcc_ref='+QuotedStr('KH6')+')) and '+qslr+
                       ' and ' + gmode +' group by state,band,qsl_r order by state,band)'+
                       'subsel join cqrlog_main main on subsel.id_cqrlog_main = main.id_cqrlog_main  order by freq,state';
                }
                l.Add('USA states')
            end
    end;
    dmData.trQ.StartTransaction;
    dmData.Q.SQL.Text := sql;
    if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;

    while not dmData.Q.Eof do
    begin
      b := dmUtils.GetBandFromFreq(dmData.Q.Fields[1].AsString);
      if (oldb <> b) then
      begin
        l.Add('');
        oldb := dmUtils.GetBandFromFreq(dmData.Q.Fields[1].AsString)
      end;


      tmp  := dmUtils.SetSize(dmData.Q.Fields[0].AsString,20) +
              dmUtils.SetSize(dmUtils.GetLabelBand(dmData.Q.Fields[1].AsString),8)+
              dmUtils.SetSize(dmData.Q.Fields[2].AsString,5)+
              dmUtils.SetSize(dmData.Q.Fields[3].AsString,5);
      l.Add(tmp);
      dmData.Q.Next
    end;
    with TfrmShowStations.Create(self) do
    try
      m.Lines.Text:= l.Text;
      ShowModal
    finally
      Free
    end
  finally
    l.Free
  end
end;

procedure TfrmWAZITUStat.ExportToHTML(htmlfile : String);
var
  f      : TextFile;
  MyCall : String ='';
  i      : Integer = 0;
  y      : integer = 0;
  tmp    : String = '';
begin
  MyCall := cqrini.ReadString('Station','Call','');

  AssignFile(f,htmlfile);
  Rewrite(f);
  Writeln(f,'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">');
  WriteLn(f,'<HTML>');
  Writeln(f,'<HEAD>');
  Writeln(f,'<META HTTP-EQUIV="CONTENT-TYPE" CONTENT="text/html; charset=utf8">');
  Writeln(f,'<TITLE> '+ Caption +' of '+MyCall+' </TITLE>');
  Writeln(f,'<META NAME="GENERATOR" CONTENT="CQRLOG ver. '+ dmData.VersionString +'">');
  Writeln(f,'<style type="text/css">');
  Writeln(f,'<!--');
  Writeln(f,'.popis {color: #FFFFFF}');
  Writeln(f,'.hlava {');
  Writeln(f,'	color: #333366;');
  Writeln(f,'	font-family: Verdana, Arial, Helvetica, sans-serif;');
  Writeln(f,'	font-size: 12px;');
  Writeln(f,'	font-weight: bold;');
  Writeln(f,'}');
  Writeln(f,'-->');
  Writeln(f,'</style>');
  Writeln(f,'</HEAD>');
  Writeln(f,'<BODY>');
  Writeln(f,'<BR>');
  Writeln(f,'<H1 ALIGN=CENTER> '+Caption+' of '+ MyCall + '</H1>');
  Writeln(f,'<BR>');
  Writeln(f,'<CENTER>');
  Writeln(f,'');

  Writeln(f,'<TABLE WIDTH="'+ IntToStr(30 + 50*(grdStat.ColCount -1)) + '" BORDER=1 CELLPADDING=2 CELLSPACING=0>');
  Writeln(f,'<COL WIDTH=30>');

  Writeln(f,'<TR VALIGN=TOP>');

  Writeln(f,'<TD WIDTH=20  bgcolor="#333366" class="hlava">');
  case StatType of
    tsWAZ,tsITU : Writeln(f,'<P><div align="center" class="popis">Zone</div></P>');
    tsWAC   : Writeln(f,'<P><div align="center" class="popis">Cont</div></P>');
    tsWAS   : Writeln(f,'<P><div align="center" class="popis">State</div></P>')
  end;
  Writeln(f,'</P>');
  Writeln(f,'</TD>');

  for i:=1 to grdStat.ColCount-1  do
  begin
    Writeln(f,'<TD WIDTH=20  bgcolor="#333366" class="hlava">');
    tmp := grdStat.Cells[i,0];
    Writeln(f,'<P><div align="center" class="popis">' + tmp +  '</div></P>');
    Writeln(f,'</TD>');
  end;  //^^ table header
  Writeln(f,'</TR>');


  for y := 1 to grdStat.RowCount-1 do
  begin
    Writeln(f,'<TR VALIGN=TOP>');
    Writeln(f,'<TD WIDTH=50  bgcolor="#333366" class="hlava">');
    Writeln(f,'<P><div align="center" class="popis">'+grdStat.Cells[0,y]);
    Writeln(f,'</div></P>');
    Writeln(f,'</TD>');
    for i := 1 to grdStat.ColCount-1 do
    begin
      Writeln(f,'<TD WIDTH=20>');
      if grdStat.Cells[i,y] = '' then
        tmp := '&nbsp;'
      else
        tmp := grdStat.Cells[i,y];
      Writeln(f,'<P ALIGN="CENTER">');
      Writeln(f,tmp);
      Writeln(f,'</P>');
      Writeln(f,'</TD>');
    end;
  end;

  Writeln(f,'</TR>');
  Writeln(f,'</TABLE>');
  Writeln(f,'</CENTER>');
  Writeln(f,'<BR>');
  Writeln(f,'<BR>');

  Writeln(f,'<!-- Hmm ... -->');

  Writeln(f,'<CENTER>');
  Writeln(f,'<TABLE WIDTH="'+ IntToStr(20 + 50*(grdSumStat.ColCount -1)) + '" BORDER=1 CELLPADDING=2 CELLSPACING=0>');
  Writeln(f,'<COL WIDTH=20>');

  Writeln(f,'<TR>');
  for i:=0 to grdSumStat.ColCount -1 do
  begin
    Writeln(f,'<TD WIDTH=20 bgcolor="#333366" class="hlava">');
    tmp := grdSumStat.Cells[i,0];
    if tmp = '' then
      tmp := '&nbsp;';
    Writeln(f,'<P><div align="center" class="popis">' + tmp +  '</div></P>');
    Writeln(f,'</TD>');
  end;  //^^ table header
  Writeln(f,'</TR>');

  Writeln(f,'<TR>');
  Writeln(f,'<TD WIDTH=20 bgcolor="#333366" class="hlava">');
  Writeln(f,'<P><div align="center" class="popis">WKD</div></P>');
  Writeln(f,'</TD>');
  for i:=1 to grdSumStat.ColCount -1 do
  begin
    Writeln(f,'<TD WIDTH=60>');
    tmp := grdSumStat.Cells[i,1];
    Writeln(f,'<P ALIGN=CENTER><FONT SIZE=2>' + tmp +  '</FONT></P>');
    Writeln(f,'</TD>');
  end;
  Writeln(f,'</TR>');

  Writeln(f,'<TR>');
  Writeln(f,'<TD WIDTH=20 bgcolor="#333366" class="hlava">');
  Writeln(f,'<P><div align="center" class="popis">CFM</div></P>');
  Writeln(f,'</TD>');
  for i:=1 to grdSumStat.ColCount -1 do
  begin
    Writeln(f,'<TD WIDTH=60>');
    tmp := grdSumStat.Cells[i,2];
    Writeln(f,'<P ALIGN=CENTER><FONT SIZE=2>' + tmp +  '</FONT></P>');
    Writeln(f,'</TD>');
  end;
  Writeln(f,'</TR>');

  Writeln(f,'</TABLE>');
  Writeln(f,'</CENTER>');
  Writeln(f,'<BR> <BR>');
  Writeln(f,'<H5 ALIGN=CENTER> <A HREF="http://www.cqrlog.com">CQRLOG ver. ' + dmData.VersionString  + ' </A></H5>');
  Writeln(f,'</BODY>');
  Writeln(f,'</HTML>');
  CloseFile(f);
end;

procedure TfrmWAZITUStat.LoadBandsSettings;
var
  i : Integer = 0;
begin
  grdStat.ColCount  := cMaxBandsCount;
  for i:=0 to cMaxBandsCount-1 do
  begin
    if dmUtils.MyBands[i][0]='' then
    begin
      grdStat.ColCount    := i+1;
      grdSumStat.ColCount := i+1;
      break
    end;
    grdStat.Cells[i+1,0] := dmUtils.MyBands[i][1];
    grdSumStat.Cells[i+1,0] := dmUtils.MyBands[i][1];
  end;
  grdStat.ColWidths[grdStat.ColCount-1] := 50;
  grdSumStat.ColWidths[grdSumStat.ColCount-1] := 50
end;

procedure TfrmWAZITUStat.CreateSummary;
var
  wkd : Word = 0;
  cfm : Word = 0;
  i,y : Integer;
begin
  grdSumStat.Cells[0,1] := 'WKD';
  grdSumStat.Cells[0,2] := 'CFM';

  for y := 1 to grdStat.ColCount-1 do
  begin
    wkd := 0;
    cfm := 0;
    for i := 1 to grdStat.RowCount-1 do
    begin
      if grdStat.Cells[y,i] <> '' then
        inc(wkd);
      case CfmType of
        tcQSL : begin
              if grdStat.Cells[y,i] = 'Q' then
                inc(cfm)
            end;
        tcQSLLoTW : begin
              if (grdStat.Cells[y,i] = 'Q') or (grdStat.Cells[y,i] = 'L') then
                inc(cfm)
            end;
        tcLoTW : begin
              if grdStat.Cells[y,i] = 'L' then
                inc(cfm)
            end
       end; //case
    end;
    grdSumStat.Cells[y,1] := IntToStr(wkd);
    grdSumStat.Cells[y,2] := IntToStr(cfm);
  end;
  grdStat.ColCount := grdStat.ColCount+1;
  grdStat.Cells[grdStat.ColCount-1,0]:= 'TOTAL';
  wkd := 0;
  cfm := 0;
  for y:=1 to grdStat.RowCount-1 do  //lines
  begin
    for i:=1 to grdStat.ColCount-1 do
    begin
      if (grdStat.Cells[i,y] = 'Q') or (grdStat.Cells[i,y] = 'L') then
        grdStat.Cells[grdStat.ColCount-1,y] := 'Q'
      else begin
        if (grdStat.Cells[grdStat.ColCount-1,y] <> 'Q') and (grdStat.Cells[i,y] = 'X') then
          grdStat.Cells[grdStat.ColCount-1,y] := 'X'
      end
    end;
    if grdStat.Cells[grdStat.ColCount-1,y] = 'Q' then
    begin
      inc(cfm);
      inc(wkd)
    end
    else begin
      if grdStat.Cells[grdStat.ColCount-1,y] = 'X' then
        inc(wkd)
    end
  end;
  grdSumStat.ColCount := grdSumStat.ColCount+1;
  grdSumStat.Cells[grdSumStat.ColCount-1,0] := 'TOTAL';
  grdSumStat.Cells[grdSumStat.ColCount-1,1] := IntToStr(wkd);
  grdSumStat.Cells[grdSumStat.ColCount-1,2] := IntToStr(cfm)
end;

procedure TfrmWAZITUStat.CreateWACStat;
const
  pAF = 1;
  pAN = 2;
  pAS = 3;
  pEU = 4;
  pNA = 5;
  pOC = 6;
  pSA = 7;

  function ContPos(cont : String) : Integer;
  begin
    if cont = 'AF' then
      Result := pAF
    else if cont = 'AN' then
      Result := pAN
    else if cont = 'AS' then
      Result := pAS
    else if cont = 'EU' then
      Result := pEU
    else if cont = 'NA' then
      Result := pNA
    else if cont = 'OC' then
      Result := pOC
    else if cont = 'SA' then
      Result := pSA
  end;

var
  i     : Integer = 1;
  y      : Integer;
  cont   : Integer;
begin

  for i:=0 to grdStat.RowCount-1 do
    for y:=0 to grdStat.ColCount-1 do
      grdStat.Cells[y,i] := '';

  LoadBandsSettings;
  Caption := 'WAC statistic';
  grdStat.Cells[0,0]:= 'Cont';

  grdStat.RowCount := 8;
  grdStat.Cells[0,pAF] := 'AF';
  grdStat.Cells[0,pAN] := 'AN';
  grdStat.Cells[0,pAS] := 'AS';
  grdStat.Cells[0,pEU] := 'EU';
  grdStat.Cells[0,pNA] := 'NA';
  grdStat.Cells[0,pOC] := 'OC';
  grdStat.Cells[0,pSA] := 'SA';

  if dmData.trQ.Active then
    dmData.trQ.Rollback;
  dmData.trQ.StartTransaction;
  if gmode = '' then
    dmData.Q.SQL.Text := 'select distinct cont,band,qsl_r from cqrlog_main where cont <> '+QuotedStr('')+' order by cont'
  else
    dmData.Q.SQL.Text := 'select distinct cont,band,qsl_r from cqrlog_main where (cont <> '+QuotedStr('')+') AND '+
                         gmode+' order by cont';
  if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
  dmData.Q.Open();
  dmData.Q.First;
  while not dmData.Q.Eof do
  begin
    cont := ContPos(dmData.Q.Fields[0].AsString);
    ShowBandChar(dmData.Q.Fields[1].AsString,cont,'X');
    dmData.Q.Next
  end;
  dmData.Q.Close;
  
  if (CfmType = tcQSLLoTW) or (CfmType = tcLoTW) then
  begin
    if gmode = '' then
      dmData.Q.SQL.Text := 'select distinct upper(cont),band from cqrlog_main where (cont <> '+QuotedStr('')+') ' +
                           ' AND (lotw_qslr='+QuotedStr('L')+') order by cont'
    else
      dmData.Q.SQL.Text := 'select distinct upper(cont),band from cqrlog_main where (cont <> '+QuotedStr('')+') AND '+
                           gmode+' AND (lotw_qslr='+QuotedStr('L')+') order by cont';
    if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      cont := ContPos(dmData.Q.Fields[0].AsString);
      ShowBandChar(dmData.Q.Fields[1].AsString,cont,'L');
      dmData.Q.Next
    end
  end;
  dmData.Q.Close;

  if (CfmType <> tcLoTW) then
  begin
    if gmode = '' then
      dmData.Q.SQL.Text := 'select distinct cont,band from cqrlog_main where (cont <> '+QuotedStr('')+') ' +
                           ' AND (qsl_r='+QuotedStr('Q')+') order by cont'
    else
      dmData.Q.SQL.Text := 'select distinct cont,band from cqrlog_main where (cont <> '+QuotedStr('')+') AND '+
                           gmode+' AND (qsl_r='+QuotedStr('Q')+') order by cont';
    if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      cont := ContPos(dmData.Q.Fields[0].AsString);
      ShowBandChar(dmData.Q.Fields[1].AsString,cont,'Q');
      dmData.Q.Next
    end
  end;
  dmData.Q.Close;
  dmData.trQ.Rollback;
  CreateSummary
end;

procedure TfrmWAZITUStat.CreateWASStat;
  function StatePos(state : String) : Integer;
  var
    i : Integer;
  begin
    Result := 1;
    for i:=1 to 50 do
      if aStates[i] = state then
      begin
        Result := i;
        break
      end
  end;

var
  i,y : Integer;
begin
  aStates[1]  := 'AK';
  aStates[2]  := 'AL';
  aStates[3]  := 'AR';
  aStates[4]  := 'AZ';
  aStates[5]  := 'CA';
  aStates[6]  := 'CO';
  aStates[7]  := 'CT';
  aStates[8]  := 'DE';
  aStates[9]  := 'FL';
  aStates[10] := 'GA';
  aStates[11] := 'HI';
  aStates[12] := 'IA';
  aStates[13] := 'ID';
  aStates[14] := 'IL';
  aStates[15] := 'IN';
  aStates[16] := 'KS';
  aStates[17] := 'KY';
  aStates[18] := 'LA';
  aStates[19] := 'MA';
  aStates[20] := 'MD';
  aStates[21] := 'ME';
  aStates[22] := 'MI';
  aStates[23] := 'MN';
  aStates[24] := 'MO';
  aStates[25] := 'MS';
  aStates[26] := 'MT';
  aStates[27] := 'NC';
  aStates[28] := 'ND';
  aStates[29] := 'NE';
  aStates[30] := 'NH';
  aStates[31] := 'NJ';
  aStates[32] := 'NM';
  aStates[33] := 'NV';
  aStates[34] := 'NY';
  aStates[35] := 'OH';
  aStates[36] := 'OK';
  aStates[37] := 'OR';
  aStates[38] := 'PA';
  aStates[39] := 'RI';
  aStates[40] := 'SC';
  aStates[41] := 'SD';
  aStates[42] := 'TN';
  aStates[43] := 'TX';
  aStates[44] := 'UT';
  aStates[45] := 'VA';
  aStates[46] := 'VT';
  aStates[47] := 'WA';
  aStates[48] := 'WI';
  aStates[49] := 'WV';
  aStates[50] := 'WY';

  for i:=0 to grdStat.RowCount-1 do
    for y:=0 to grdStat.ColCount-1 do
      grdStat.Cells[y,i] := '';

  grdStat.RowCount := 51;

  grdStat.Cells[0,0]  := 'State';

  for i:=1 to 50 do
    grdStat.Cells[0,i] := dmUtils.USstates[i];

  LoadBandsSettings;
  Caption := 'WAS statistic';

  if dmData.trQ.Active then
    dmData.trQ.Rollback;
  dmData.trQ.StartTransaction;
  if gmode = '' then
    dmData.Q.SQL.Text := 'select distinct state,band,qsl_r from cqrlog_main where state <> '+QuotedStr('')+
                         ' and ((adif=291) or (adif=6)'+
                         ' or (adif=110)) order by state'
  else
    dmData.Q.SQL.Text := 'select distinct state,band,qsl_r from cqrlog_main where (state <> '+QuotedStr('')+')'+
                         ' and ((adif=291) or (adif=6)'+
                         ' or (adif=110)) and '+gmode+' order by state';
  if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
  dmData.Q.Open();
  dmData.Q.First;
  while not dmData.Q.Eof do
  begin
    if trim(dmData.Q.Fields[0].AsString) <> '' then
      ShowBandChar(dmData.Q.Fields[1].AsString,StatePos(dmData.Q.Fields[0].AsString),'X');
    dmData.Q.Next
  end;
  dmData.Q.Close;

  if (CfmType = tcQSLLoTW) or (CfmType = tcLoTW) then
  begin
    if gmode = '' then
      dmData.Q.SQL.Text := 'select distinct state,band from cqrlog_main where (state <> '+QuotedStr('')+') ' +
                           ' AND (lotw_qslr='+QuotedStr('L')+')'+
                           ' and ((adif=291) or (adif=6)'+
                           ' or (adif=110)) order by state'

    else
      dmData.Q.SQL.Text := 'select distinct state,band from cqrlog_main where (state <> '+QuotedStr('')+')'+
                           ' AND (lotw_qslr='+QuotedStr('L')+')'+
                           ' and ((adif=291) or (adif=6)'+
                           ' or (adif=110)) and '+gmode+' order by state';

    if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      if trim(dmData.Q.Fields[0].AsString) <> '' then
        ShowBandChar(dmData.Q.Fields[1].AsString,StatePos(dmData.Q.Fields[0].AsString),'L');
      dmData.Q.Next
    end
  end;
  dmData.Q.Close;

  if (CfmType <> tcLoTW) then
  begin
    if gmode = '' then
      dmData.Q.SQL.Text := 'select distinct state,band from cqrlog_main where (state <> '+QuotedStr('')+') ' +
                           ' AND (qsl_r='+QuotedStr('Q')+')'+
                           ' and ((adif=291) or (adif=6)'+
                           ' or (adif=110)) order by state'

    else
      dmData.Q.SQL.Text := 'select distinct state,band from cqrlog_main where (state <> '+QuotedStr('')+')'+
                           ' and (qsl_r='+QuotedStr('Q')+')'+
                           ' and ((adif=291) or (adif=6)'+
                           ' or (adif=110)) and '+gmode+' order by state';

    if dmData.DebugLevel>=1 then Writeln(dmData.Q.SQL.Text);
    dmData.Q.Open();
    dmData.Q.First;
    while not dmData.Q.Eof do
    begin
      if trim(dmData.Q.Fields[0].AsString) <> '' then
        ShowBandChar(dmData.Q.Fields[1].AsString,StatePos(dmData.Q.Fields[0].AsString),'Q');
      dmData.Q.Next
    end
  end;
  dmData.Q.Close;
  dmData.trQ.Rollback;
  CreateSummary

{
AK  	Alaska
AL 	Alabama
AR 	Arkansas
AZ 	Arizona
CA 	California
CO 	Colorado
CT 	Connecticut
DC 	Dist Of Col
DE 	Delaware
FL 	Florida
GA 	Georgia
HI 	Hawaii
IA 	Iowa
ID 	Idaho
IL 	Illinois
IN 	Indiana
KS 	Kansas
KY 	Kentucky
LA 	Louisiana
MA 	Massachusetts
MD 	Maryland
ME 	Maine
MI 	Michigan
MN 	Minnesota
MO 	Missouri
MS 	Mississippi
MT 	Montana
NC 	North Carolina
ND 	North Dakota
NE 	Nebraska
NH 	New Hampshire
NJ 	New Jersey
NM 	New Mexico
NV 	Nevada
NY 	New York
OH 	Ohio
OK 	Oklahoma
OR 	Oregon
PA 	Pennsylvania
RI 	Rhode Island
SC 	South Carolina
SD 	South Dakota
TN 	Tennessee
TX 	Texas
UT 	Utah
VA 	Virginia
VT 	Vermont
WA 	Washington
WI 	Wisconsin
WV 	West Virginia
WY 	Wyoming

}
end;

initialization
  {$I fWAZITUStat.lrs}

end.

